@@ -16,6 +16,7 @@ import json
 import os
 import pathlib
 import re
+import ssl
 import tempfile
 import typing as t
 import warnings
@@ -24,7 +25,8 @@ from types import FrameType
 
 import __main__
 import markdown as md_lib
-from flask import Blueprint, Flask, request, send_from_directory
+import tzlocal
+from flask import Blueprint, Flask, g, jsonify, request, send_from_directory
 from werkzeug.utils import secure_filename
 
 if util.find_spec("pyngrok"):
@@ -36,6 +38,7 @@ from .config import Config, ConfigParameter, _Config
 from .data.content_accessor import _ContentAccessor
 from .data.data_accessor import _DataAccessor, _DataAccessors
 from .data.data_format import _DataFormat
+from .data.data_scope import _DataScopes
 from .page import Page
 from .partial import Partial
 from .renderers import _EmptyPage
@@ -46,12 +49,15 @@ from .state import State
 from .types import _WsType
 from .utils import (
     _delscopeattr,
+    _filter_locals,
     _get_client_var_name,
+    _get_module_name_from_frame,
     _get_non_existent_file_path,
     _getscopeattr,
     _getscopeattr_drill,
     _hasscopeattr,
     _is_in_notebook,
+    _LocalsContext,
     _MapDict,
     _setscopeattr,
     _setscopeattr_drill,
@@ -61,10 +67,13 @@ from .utils import (
     _TaipyData,
     _TaipyLov,
     _TaipyLovValue,
+    _variable_decode,
+    _variable_encode,
 )
 from .utils._adapter import _Adapter
 from .utils._bindings import _Bindings
 from .utils._evaluator import _Evaluator
+from .utils._variable_directory import _RE_TPMDL_DECODE, _VariableDirectory
 
 
 class Gui:
@@ -74,14 +83,17 @@ class Gui:
 
         on_action (Callable): The function that is called when a control
             triggers an action, as the result of an interaction with the end-user.<br/>
-            It defaults to the _on_action_ global function defined in the Python
+            It defaults to the `on_action()` global function defined in the Python
             application. If there is no such function, actions will not trigger anything.
         on_change (Callable): The function that is called when a control
             modifies variables it is bound to, as the result of an interaction with the
             end-user.<br/>
-            It defaults to the _on_change_ global function defined in the Python
+            It defaults to the `on_change()` global function defined in the Python
             application. If there is no such function, user interactions will not trigger
             anything.
+        on_init (Callable): The function that is called on the first connection of a new user.<br/>
+            It defaults to the `on_init()` global function defined in the Python
+            application.
         state (State^): **Only defined when running in an IPython notebook context.**<br/>
             The unique instance of `State^` that you can use to change bound variables
             directly, potentially impacting the interface in real-time.
@@ -99,6 +111,7 @@ class Gui:
     __env_filename = "taipy.gui.env"
     __UI_BLOCK_NAME = "TaipyUiBlockVar"
     __MESSAGE_GROUPING_NAME = "TaipyMessageGrouping"
+    __ON_INIT_NAME = "TaipyOnInit"
     __CONTENT_ROOT = "/taipy-content/"
     __UPLOAD_URL = "/taipy-uploads"
 
@@ -109,13 +122,13 @@ class Gui:
     __reserved_routes: t.List[str] = ["taipy-init", "taipy-jsx", "taipy-content", "taipy-uploads"]
     _aggregate_functions: t.List[str] = ["count", "sum", "mean", "median", "min", "max", "std", "first", "last"]
 
+    __LOCAL_TZ = str(tzlocal.get_localzone())
+
     def __init__(
         self,
         page: t.Optional[t.Union[str, Page]] = None,
         pages: t.Optional[dict] = None,
-        css_file: str = os.path.splitext(os.path.basename(__main__.__file__))[0]
-        if hasattr(__main__, "__file__")
-        else "Taipy",
+        css_file: t.Optional[str] = None,
         path_mapping: t.Optional[dict] = {},
         env_filename: t.Optional[str] = None,
         flask: t.Optional[Flask] = None,
@@ -160,10 +173,13 @@ class Gui:
         """
         # store suspected local containing frame
         self.__frame = t.cast(FrameType, t.cast(FrameType, inspect.currentframe()).f_back)
+        self.__default_module_name = _get_module_name_from_frame(self.__frame)
 
         # Preserve server config for server initialization
         self._path_mapping = path_mapping
         self._flask = flask
+        if css_file is None:
+            css_file = os.path.splitext(os.path.basename(self.__frame.f_code.co_filename))[0] or "Taipy"
         self._css_file = css_file
 
         self._config = _Config()
@@ -171,15 +187,20 @@ class Gui:
         self._accessors = _DataAccessors()
         self.__state: t.Optional[State] = None
         self.__bindings = _Bindings(self)
-        self.__locals_bind: t.Dict[str, t.Any] = {}
+        self.__locals_context = _LocalsContext()
+        self.__var_dir = _VariableDirectory(self.__locals_context)
 
         self.__evaluator: _Evaluator = None  # type: ignore
         self.__adapter = _Adapter()
         self.__directory_name_of_pages: t.List[str] = []
 
         # default actions
-        self.on_change: t.Optional[t.Callable] = None
         self.on_action: t.Optional[t.Callable] = None
+        self.on_change: t.Optional[t.Callable] = None
+        self.on_init: t.Optional[t.Callable] = None
+
+        # sid from client_id
+        self.__client_id_2_sid: t.Dict[str, t.Set[str]] = {}
 
         # Load default config
         self._flask_blueprint: t.List[Blueprint] = []
@@ -217,6 +238,9 @@ class Gui:
     def _bindings(self):
         return self.__bindings
 
+    def _get_data_scope(self):
+        return self.__bindings._get_data_scope()
+
     def _get_config(self, name: ConfigParameter, default_value: t.Any) -> t.Any:
         return self._config._get_config(name, default_value)
 
@@ -238,9 +262,59 @@ class Gui:
     def _bind(self, name: str, value: t.Any) -> None:
         self._bindings()._bind(name, value)
 
+    def __get_state(self):
+        return self.__state
+
+    def _get_client_id(self) -> str:
+        return (
+            _DataScopes._GLOBAL_ID if self._bindings()._get_single_client() else getattr(g, "client_id", "unknown id")
+        )
+
+    def __set_client_id_in_context(self, client_id: t.Optional[str] = None):
+        if not client_id and request:
+            client_id = request.args.get("client_id", "")
+        if client_id and request:
+            if sid := getattr(request, "sid", None):
+                sids = self.__client_id_2_sid.get(client_id, None)
+                if sids is None:
+                    sids = set()
+                    self.__client_id_2_sid[client_id] = sids
+                sids.add(sid)
+        g.client_id = client_id
+
+    def __is_var_modified_in_context(self, var_name: str, derived_vars: t.Set[str]) -> bool:
+        modified_vars: t.Optional[t.Set[str]] = getattr(g, "modified_vars", None)
+        der_vars: t.Optional[t.Set[str]] = getattr(g, "derived_vars", None)
+        setattr(g, "update_count", getattr(g, "update_count", 0) + 1)
+        if modified_vars is None:
+            modified_vars = set()
+            g.modified_vars = modified_vars
+        if der_vars is None:
+            g.derived_vars = derived_vars
+        else:
+            der_vars.update(derived_vars)
+        if var_name in modified_vars:
+            return True
+        else:
+            modified_vars.add(var_name)
+            return False
+
+    def __clean_vars_on_exit(self) -> t.Optional[t.Set[str]]:
+        update_count = getattr(g, "update_count", 0) - 1
+        if update_count < 1:
+            vars: t.Set[str] = getattr(g, "derived_vars", set())
+            delattr(g, "update_count")
+            delattr(g, "modified_vars")
+            delattr(g, "derived_vars")
+            return vars
+        else:
+            setattr(g, "update_count", update_count)
+            return None
+
     def _manage_message(self, msg_type: _WsType, message: dict) -> None:
         try:
-            self._bindings()._set_client_id(message)
+            self.__set_client_id_in_context(message.get("client_id"))
+            self._set_locals_context(message.get("module_context") or None)
             if msg_type == _WsType.UPDATE.value:
                 payload = message.get("payload", {})
                 self.__front_end_update(
@@ -258,6 +332,7 @@ class Gui:
                 self.__request_var_update(message.get("payload"))
             elif msg_type == _WsType.CLIENT_ID.value:
                 self._bindings()._get_or_create_scope(message.get("payload", ""))
+            self._reset_locals_context()
         except Exception as e:
             warnings.warn(f"Decoding Message has failed: {message}\n{e}")
 
@@ -298,43 +373,68 @@ class Gui:
         propagate=True,
         holder: t.Optional[_TaipyBase] = None,
         on_change: t.Optional[str] = None,
+        from_map_dict: bool = False,
     ) -> None:
+        if from_map_dict:
+            var_name = _variable_encode(var_name, self._get_locals_context())
         if holder:
             var_name = holder.get_name()
         hash_expr = self.__evaluator.get_hash_from_expr(var_name)
-        modified_vars = {hash_expr}
+        derived_vars = {hash_expr}
         # Use custom attrsetter function to allow value binding for _MapDict
         if propagate:
             _setscopeattr_drill(self, hash_expr, value)
             # In case expression == hash (which is when there is only a single variable in expression)
-            if var_name == hash_expr:
-                modified_vars.update(self._re_evaluate_expr(var_name))
+            if var_name == hash_expr or hash_expr.startswith("tpec_"):
+                derived_vars.update(self._re_evaluate_expr(var_name))
         elif holder:
-            modified_vars.update(self._evaluate_holders(hash_expr))
-        self.__call_on_change(
-            var_name,
-            value.get() if isinstance(value, _TaipyBase) else value._dict if isinstance(value, _MapDict) else value,
-            on_change,
-        )
-        self.__send_var_list_update(list(modified_vars), var_name)
+            derived_vars.update(self._evaluate_holders(hash_expr))
+        # if the variable has been evaluated then skip updating to prevent infinite loop
+        var_modified = self.__is_var_modified_in_context(hash_expr, derived_vars)
+        if not var_modified:
+            self.__call_on_change(
+                var_name,
+                value.get() if isinstance(value, _TaipyBase) else value._dict if isinstance(value, _MapDict) else value,
+                on_change,
+            )
+        derived_modified = self.__clean_vars_on_exit()
+        if derived_modified is not None:
+            self.__send_var_list_update(list(derived_modified), var_name)
 
     def __call_on_change(self, var_name: str, value: t.Any, on_change: t.Optional[str] = None):
         # TODO: what if _update_function changes 'var_name'... infinite loop?
-        on_change_fn = None
-        if on_change:
-            on_change_fn = self._get_user_function(on_change)
-        if not callable(on_change_fn) and hasattr(self, "on_change") and callable(self.on_change):
-            on_change_fn = self.on_change
+        var_name_decode, module_name = _variable_decode(self._get_expr_from_hash(var_name))
+        current_context = self._get_locals_context()
+        if module_name == current_context:
+            var_name = var_name_decode
+        else:
+            if var_name not in self.__var_dir._var_head:
+                warnings.warn(f"Can't find matching variable for {var_name} on {current_context} context")
+                return
+            _found = False
+            for k, v in self.__var_dir._var_head[var_name]:
+                if v == current_context:
+                    var_name = k
+                    _found = True
+                    break
+            if not _found:
+                warnings.warn(f"Can't find matching variable for {var_name} on {current_context} context")
+                return
+        on_change_fn = self._get_user_function(on_change) if on_change else None
+        if not callable(on_change_fn):
+            on_change_fn = self._get_user_function("on_change")
         if callable(on_change_fn):
             try:
                 argcount = on_change_fn.__code__.co_argcount
                 args: t.List[t.Any] = [None for _ in range(argcount)]
                 if argcount > 0:
-                    args[0] = self.__state
+                    args[0] = self.__get_state()
                 if argcount > 1:
                     args[1] = var_name
                 if argcount > 2:
                     args[2] = value
+                if argcount > 3:
+                    args[3] = current_context
                 on_change_fn(*args)
             except Exception as e:
                 warnings.warn(f"{on_change or 'on_change'}: callback function raised an exception: {e}")
@@ -344,6 +444,7 @@ class Gui:
         return Gui.__CONTENT_ROOT + ret_value[0] if isinstance(ret_value, tuple) else ret_value
 
     def __serve_content(self, path: str) -> t.Any:
+        self.__set_client_id_in_context()
         parts = path.split("/")
         if len(parts) > 1:
             file_name = parts[-1]
@@ -355,6 +456,7 @@ class Gui:
         return ("", 404)
 
     def __upload_files(self):
+        self.__set_client_id_in_context()
         if "var_name" not in request.form:
             warnings.warn("No var name")
             return ("No var name", 400)
@@ -540,7 +642,13 @@ class Gui:
         self.__send_ws({"type": _WsType.MULTIPLE_UPDATE.value, "payload": payload})
 
     def __get_ws_receiver(self) -> t.Union[t.List[str], t.Any, None]:
-        return None if self._bindings()._get_single_client() else getattr(request, "sid", None)
+        if self._bindings()._get_single_client():
+            return None
+        sid = getattr(request, "sid", None) if request else None
+        sids = self.__client_id_2_sid.get(self._get_client_id(), set())
+        if sid:
+            sids.add(sid)
+        return list(sids)
 
     def __get_message_grouping(self):
         if not _hasscopeattr(self, Gui.__MESSAGE_GROUPING_NAME):
@@ -575,7 +683,9 @@ class Gui:
     def _get_user_function(self, func_name: str) -> t.Union[t.Callable, str]:
         func = _getscopeattr(self, func_name, None)
         if not callable(func):
-            func = self.__locals_bind.get(func_name)
+            func = self._get_locals_bind().get(func_name)
+        if not callable(func):
+            func = self.__locals_context.get_default().get(func_name)
         if callable(func):
             return func
         return func_name
@@ -603,7 +713,7 @@ class Gui:
                 argcount = action_function.__code__.co_argcount
                 args = [None for _ in range(argcount)]
                 if argcount > 0:
-                    args[0] = self.__state
+                    args[0] = self.__get_state()
                 if argcount > 1:
                     args[1] = id
                 if argcount > 2:
@@ -617,7 +727,7 @@ class Gui:
         return False
 
     def _call_function_with_state(self, user_function: t.Callable, args: t.List[t.Any]) -> t.Any:
-        args.insert(0, self.__state)
+        args.insert(0, self.__get_state())
         arg_count = user_function.__code__.co_argcount
         if arg_count > len(args):
             args += (arg_count - len(args)) * [None]
@@ -625,6 +735,15 @@ class Gui:
             args = args[:arg_count]
         return user_function(*args)
 
+    def _call_user_callback(self, context_id: t.Optional[str], user_callback: t.Callable, args: t.List[t.Any]) -> t.Any:
+        try:
+            with self.get_flask_app().app_context():
+                self.__set_client_id_in_context(context_id)
+                return self._call_function_with_state(user_callback, args)
+        except Exception as e:
+            warnings.warn(f"invoke_state_callback: '{user_callback.__name__}' function invocation exception: {e}")
+        return None
+
     # Proxy methods for Evaluator
     def _evaluate_expr(self, expr: str) -> t.Any:
         return self.__evaluator.evaluate_expr(self, expr)
@@ -654,6 +773,9 @@ class Gui:
     def _get_adapter_for_type(self, type_name: str) -> t.Optional[t.Callable]:
         return self.__adapter._get_for_type(type_name)
 
+    def _get_unique_type_adapter(self, type_name: str) -> str:
+        return self.__adapter._get_unique_type(type_name)
+
     def _run_adapter(
         self, adapter: t.Optional[t.Callable], value: t.Any, var_name: str, id_only=False
     ) -> t.Union[t.Tuple[str, ...], str, None]:
@@ -679,25 +801,48 @@ class Gui:
             from .renderers import Html, Markdown
 
             if re_match := Gui.__RE_HTML.match(file_name):
-                renderers = Html(os.path.join(folder_path, file_name))
+                renderers = Html(os.path.join(folder_path, file_name), frame=None)
                 renderers.modify_taipy_base_url(folder_name)
                 self.add_page(name=f"{folder_name}/{re_match.group(1)}", page=renderers)
             elif re_match := Gui.__RE_MD.match(file_name):
-                renderers_md = Markdown(os.path.join(folder_path, file_name))
+                renderers_md = Markdown(os.path.join(folder_path, file_name), frame=None)
                 self.add_page(name=f"{folder_name}/{re_match.group(1)}", page=renderers_md)
             elif os.path.isdir(child_dir_path := os.path.join(folder_path, file_name)):
                 child_dir_name = f"{folder_name}/{file_name}"
                 self.__add_pages_in_folder(child_dir_name, child_dir_path)
 
-    def _get_locals_bind(self):
-        return self.__locals_bind
+    # Proxy methods for LocalsContext
+    def _get_locals_bind(self) -> t.Dict[str, t.Any]:
+        return self.__locals_context.get_locals()
 
-    def _get_root_page_name(self):
-        return self.__root_page_name
+    def _get_locals_context(self) -> str:
+        current_context = self.__locals_context.get_context()
+        return current_context if current_context is not None else self.__default_module_name
+
+    def _set_locals_context(self, context: t.Optional[str]) -> None:
+        self.__locals_context.set_locals_context(context)
+
+    def _reset_locals_context(self) -> None:
+        self.__locals_context.reset_locals_context()
+
+    @staticmethod
+    def _get_root_page_name():
+        return Gui.__root_page_name
 
     def _set_flask(self, flask: Flask):
         self._flask = flask
 
+    def _get_default_module_name(self):
+        return self.__default_module_name
+
+    @staticmethod
+    def _get_timezone() -> str:
+        return Gui.__LOCAL_TZ
+
+    @staticmethod
+    def _set_timezone(tz: str):
+        Gui.__LOCAL_TZ = tz
+
     # Public methods
     def add_page(
         self,
@@ -718,6 +863,7 @@ class Gui:
                 - If it is not, the page content is read from this string as
                   Markdown text.
             style (Optional[str]): Additional CSS style to apply to this page.
+
                 - if there is style associated with a page, it is used at a global level
                 - if there is no style associated with the page, the style is cleared at a global level
                 - if the page is embedded in a block control, the style is ignored
@@ -739,7 +885,7 @@ class Gui:
         if isinstance(page, str):
             from .renderers import Markdown
 
-            page = Markdown(page)
+            page = Markdown(page, frame=None)
         elif not isinstance(page, Page):
             raise Exception(f'"page" is invalid for page name "{name if name != Gui.__root_page_name else "/"}')
         # Init a new page
@@ -750,8 +896,12 @@ class Gui:
         # Append page to _config
         self._config.pages.append(new_page)
         self._config.routes.append(name)
+        # Update locals context
+        self.__locals_context.add(page._get_module_name(), page._get_locals())
+        # Update variable directory
+        self.__var_dir.add_frame(page._frame)
 
-    def add_pages(self, pages: t.Union[dict[str, t.Union[str, Page]], str] = None) -> None:
+    def add_pages(self, pages: t.Union[t.Dict[str, t.Union[str, Page]], str] = None) -> None:
         """Add several pages to the graphical User Interface.
 
         Arguments:
@@ -784,10 +934,10 @@ class Gui:
 
             For every file that is found:
 
-                - If the filename extention is _.md_, it is read as Markdown content and
-                  a new page is created with the base name of this filename.
-                - If the filename extention is _.html_, it is read as HTML content and
-                  a new page is created with the base name of this filename.
+            - If the filename extention is _.md_, it is read as Markdown content and
+             a new page is created with the base name of this filename.
+            - If the filename extention is _.html_, it is read as HTML content and
+             a new page is created with the base name of this filename.
 
             For example, say you have the following directory structure:
             ```
@@ -872,13 +1022,17 @@ class Gui:
         if isinstance(page, str):
             from .renderers import Markdown
 
-            page = Markdown(page)
+            page = Markdown(page, frame=None)
         elif not isinstance(page, Page):
             raise Exception(f'Partial name "{new_partial._route}" has invalid Page')
         new_partial._renderer = page
         # Append partial to _config
         self._config.partials.append(new_partial)
         self._config.partial_routes.append(str(new_partial._route))
+        # Update locals context
+        self.__locals_context.add(page._get_module_name(), page._get_locals())
+        # Update variable directory
+        self.__var_dir.add_frame(page._frame)
         return new_partial
 
     def _update_partial(self, partial: Partial):
@@ -895,13 +1049,15 @@ class Gui:
         return partial
 
     # Main binding method (bind in markdown declaration)
-    def _bind_var(self, var_name: str) -> bool:
-        if not hasattr(self._bindings(), var_name) and var_name in self.__locals_bind.keys():
-            self._bind(var_name, self.__locals_bind[var_name])
-            return True
-        return False
+    def _bind_var(self, var_name: str) -> str:
+        encoded_var_name = self.__var_dir.add_var(var_name, self._get_locals_context())
+        if not hasattr(self._bindings(), encoded_var_name) and var_name in self._get_locals_bind().keys():
+            self._bind(encoded_var_name, self._get_locals_bind()[var_name])
+        return encoded_var_name
 
     def _bind_var_val(self, var_name: str, value: t.Any) -> bool:
+        if not _RE_TPMDL_DECODE.match(var_name):
+            var_name = self.__var_dir.add_var(var_name, self._get_locals_context())
         if not hasattr(self._bindings(), var_name):
             self._bind(var_name, value)
             return True
@@ -915,7 +1071,7 @@ class Gui:
             )
             func = None
         if func is None:
-            func = self.__locals_bind.get(name)
+            func = self._get_locals_bind().get(name)
         if func is not None:
             if callable(func):
                 setattr(self, name, func)
@@ -971,19 +1127,110 @@ class Gui:
             return
         self.__send_ws_navigate(to)
 
+    def __init_route(self):
+        self.__set_client_id_in_context()
+        if hasattr(self, "on_init") and callable(self.on_init):
+            if not _hasscopeattr(self, Gui.__ON_INIT_NAME):
+                _setscopeattr(self, Gui.__ON_INIT_NAME, True)
+                try:
+                    self._call_function_with_state(self.on_init, [])
+                except Exception as e:
+                    warnings.warn(f"Exception on on_init execution \n{e}")
+        return self._render_route()
+
+    def __render_page(self, page_name: str) -> t.Any:
+        self.__set_client_id_in_context()
+        page = next((page_i for page_i in self._config.pages if page_i._route == page_name), None)
+
+        # try partials
+        if page is None:
+            page = self._get_partial(page_name)
+        # Make sure that there is a page instance found
+        if page is None:
+            return (jsonify({"error": "Page doesn't exist!"}), 400, {"Content-Type": "application/json; charset=utf-8"})
+        context = page.render(self)
+        if (
+            page_name == Gui.__root_page_name
+            and page._rendered_jsx is not None
+            and "<PageContent" not in page._rendered_jsx
+        ):
+            page._rendered_jsx += "<PageContent />"
+        # Return jsx page
+        if page._rendered_jsx is not None:
+            return self._server._render(
+                page._rendered_jsx, page._style if page._style is not None else "", page._head, context
+            )
+        else:
+            return ("No page template", 404)
+
+    def _render_route(self) -> t.Any:
+        router = '<Routes key="routes">'
+        router += (
+            '<Route path="/" key="'
+            + Gui.__root_page_name
+            + '" element={<MainPage key="tr'
+            + Gui.__root_page_name
+            + '" path="/'
+            + Gui.__root_page_name
+            + '"'
+        )
+        routes = self._config.routes
+        route = next((r for r in routes if r != Gui.__root_page_name), None)
+        router += (' route="/' + route + '"') if route else ""
+        router += " />} >"
+        locations = {"/": f"/{Gui.__root_page_name}"}
+        for route in routes:
+            if route != Gui.__root_page_name:
+                router += (
+                    '<Route path="'
+                    + route
+                    + '" key="'
+                    + route
+                    + '" element={<TaipyRendered key="tr'
+                    + route
+                    + '"/>} />'
+                )
+                locations[f"/{route}"] = f"/{route}"
+        router += '<Route path="*" key="NotFound" element={<NotFound404 />} />'
+        router += "</Route>"
+        router += "</Routes>"
+
+        return self._server._direct_render_json(
+            {
+                "router": router,
+                "locations": locations,
+                "blockUI": self._is_ui_blocked(),
+            }
+        )
+
     def _register_data_accessor(self, data_accessor_class: t.Type[_DataAccessor]) -> None:
         self._accessors._register(data_accessor_class)
 
     def get_flask_app(self) -> Flask:
-        """Get the internal Flask application"""
+        """Get the internal Flask application.
+
+        Returns:
+            the Flask instance used.
+        """
         return self._server.get_flask()
 
     def _set_frame(self, frame: FrameType):
         if not isinstance(frame, FrameType):
             raise RuntimeError("frame must be a FrameType where Gui can collect the local variables.")
         self.__frame = frame
+        self.__default_module_name = _get_module_name_from_frame(self.__frame)
 
-    def run(self, run_server: bool = True, run_in_thread: bool = False, **kwargs) -> t.Optional[Flask]:
+    def _set_state(self, state: State):
+        if isinstance(state, State):
+            self.__state = state
+
+    def run(
+        self,
+        run_server: bool = True,
+        run_in_thread: bool = False,
+        ssl_context: t.Optional[t.Union[ssl.SSLContext, t.Tuple[str, t.Optional[str]], t.Literal["adhoc"]]] = None,
+        **kwargs,
+    ) -> t.Optional[Flask]:
         """
         Starts the server that delivers pages to Web clients.
 
@@ -1001,10 +1248,17 @@ class Gui:
                 If set to _True_, the Web server is run is a separated thread.
                 Note that if you are running in an IPython notebook context, the Web
                 server is always run in a separate thread.
-            kwargs: Additional keywords that configure how this `Gui` is run.
+            ssl_context (Optional[Union[ssl.SSLContext, Tuple[str, Optional[str]], te.Literal['adhoc']]]):
+                Configure TLS to serve over HTTPS. Can be an ssl.SSLContext object, a (cert_file, key_file) tuple to
+                create a typical context, or the string 'adhoc' to generate a temporary self-signed certificate.</br>
+                The default value is None.
+            **kwargs: Additional keyword arguments that configure how this `Gui` is run.
                 Please refer to the
                 [Configuration](../gui/configuration.md#configuring-the-gui-instance)
                 section in the User Manual for more information.
+
+        Returns:
+            The Flask instance if _run_server_ is _False_ else _None_.
         """
 
         app_config = self._config.config
@@ -1056,28 +1310,31 @@ class Gui:
             app_config["use_reloader"] = False
             print(f" * NGROK Public Url: {http_tunnel.public_url}")
 
-        # Save all local variables of the parent frame (usually __main__)
-        if isinstance(kwargs.get("locals_bind"), dict):
-            self.__locals_bind = kwargs.get("locals_bind")  # type: ignore
-            warnings.warn("Caution: the Gui instance is using a custom 'locals_bind' setting.")
-        else:
-            self.__locals_bind = self.__frame.f_locals
+        locals_bind = _filter_locals(self.__frame.f_locals)
 
-        self.__state = State(self, self.__locals_bind.keys())
+        self.__locals_context.set_default(locals_bind)
+
+        self.__var_dir.set_default(self.__frame)
+
+        if self.__state is None:
+            self.__state = State(self, self.__locals_context.get_all_keys())
 
         if _is_in_notebook():
             # to allow gui.state.x in notebook mode
             self.state = self.__state
 
+        with self.get_flask_app().app_context():
+            self.__var_dir.process_imported_var()
+            # bind on_change and on_action function if available
+            self.__bind_local_func("on_init")
+            self.__bind_local_func("on_change")
+            self.__bind_local_func("on_action")
+
         # base global ctx is TaipyHolder classes + script modules and callables
         glob_ctx = {t.__name__: t for t in _TaipyBase.__subclasses__()}
-        glob_ctx.update({k: v for k, v in self.__locals_bind.items() if inspect.ismodule(v) or callable(v)})
+        glob_ctx.update({k: v for k, v in locals_bind.items() if inspect.ismodule(v) or callable(v)})
         self.__evaluator = _Evaluator(glob_ctx)
 
-        # bind on_change and on_action function if available
-        self.__bind_local_func("on_change")
-        self.__bind_local_func("on_action")
-
         # add en empty main page if it is not defined
         if Gui.__root_page_name not in self._config.routes:
             new_page = _Page()
@@ -1107,19 +1364,15 @@ class Gui:
                 template_folder=f"{_absolute_path}{os.path.sep}webapp",
                 title=self._get_config("title", "Taipy App"),
                 favicon=self._get_config("favicon", "/favicon.png"),
-                themes=self._get_themes(),
                 root_margin=self._get_config("margin", None),
             )
         )
 
         # Run parse markdown to force variables binding at runtime
-        # (save rendered html to page.rendered_jsx for optimization)
-        for page in self._config.pages + self._config.partials:  # type: ignore
-            # Server URL Rule for each page jsx
-            pages_bp.add_url_rule(f"/taipy-jsx/{page._route}/", view_func=self._server._render_page)
+        pages_bp.add_url_rule("/taipy-jsx/<path:page_name>", view_func=self.__render_page)
 
         # server URL Rule for flask rendered react-router
-        pages_bp.add_url_rule("/taipy-init/", view_func=self._server._render_route)
+        pages_bp.add_url_rule("/taipy-init", view_func=self.__init_route)
 
         # Register Flask Blueprint if available
         for bp in self._flask_blueprint:
@@ -1142,6 +1395,7 @@ class Gui:
             use_reloader=app_config["use_reloader"],
             flask_log=app_config["flask_log"],
             run_in_thread=run_in_thread,
+            ssl_context=ssl_context,
         )
 
     def stop(self):
