@@ -62,7 +62,7 @@ class _Factory:
         .set_attributes(
             [
                 ("id",),
-                ("on_action", _AttributeType.function, ""),
+                ("on_action", _AttributeType.function),
                 ("active", _AttributeType.dynamic_boolean, True),
                 ("hover_text", _AttributeType.dynamic_string),
             ]
@@ -87,7 +87,7 @@ class _Factory:
                 ("limit_rows", _AttributeType.boolean),
                 ("render", _AttributeType.dynamic_boolean, True),
                 ("hover_text", _AttributeType.dynamic_string),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
             ]
         )
         .get_chart_config("scatter", "lines+markers")
@@ -111,7 +111,7 @@ class _Factory:
                 ("active", _AttributeType.dynamic_boolean, True),
                 ("editable", _AttributeType.dynamic_boolean, True),
                 ("hover_text", _AttributeType.dynamic_string),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
             ]
         )
         .set_propagate(),
@@ -201,7 +201,7 @@ class _Factory:
         .set_attributes(
             [
                 ("id",),
-                ("on_action", _AttributeType.function, ""),
+                ("on_action", _AttributeType.function),
                 ("active", _AttributeType.dynamic_boolean, True),
                 ("width",),
                 ("height",),
@@ -224,6 +224,8 @@ class _Factory:
                 ("format",),
                 ("orientation"),
                 ("hover_text", _AttributeType.dynamic_string),
+                ("width",),
+                ("height",),
             ]
         ),
         "input": lambda gui, control_type, attrs: _Builder(
@@ -232,15 +234,19 @@ class _Factory:
             element_name="Input",
             attributes=attrs,
         )
-        .set_type("text")
+        .set_input_type("text", True)
         .set_value_and_default()
+        .set_change_delay()
         .set_propagate()
         .set_attributes(
             [
                 ("id",),
                 ("active", _AttributeType.dynamic_boolean, True),
                 ("hover_text", _AttributeType.dynamic_string),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
+                ("on_action", _AttributeType.function),
+                ("action_keys",),
+                ("label",),
             ]
         ),
         "layout": lambda gui, control_type, attrs: _Builder(
@@ -269,7 +275,7 @@ class _Factory:
                 ("label"),
                 ("width"),
                 ("width[mobile]",),
-                ("on_action", _AttributeType.function, "on_menu_action"),
+                ("on_action", _AttributeType.function),
                 ("inactive_ids", _AttributeType.dynamic_list),
                 ("hover_text", _AttributeType.dynamic_string),
             ]
@@ -295,15 +301,18 @@ class _Factory:
             attributes=attrs,
             default_value=0,
         )
-        .set_type("number")
-        .set_value_and_default()
+        .set_input_type("number")
+        .set_value_and_default(var_type=_AttributeType.dynamic_number)
+        .set_change_delay()
         .set_propagate()
         .set_attributes(
             [
                 ("id",),
                 ("active", _AttributeType.dynamic_boolean, True),
                 ("hover_text", _AttributeType.dynamic_string),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
+                ("on_action", _AttributeType.function),
+                ("label",),
             ]
         ),
         "pane": lambda gui, control_type, attrs: _Builder(
@@ -322,7 +331,7 @@ class _Factory:
                 ("width", _AttributeType.string_or_number, "30vw"),
                 ("height", _AttributeType.string_or_number, "30vh"),
                 ("hover_text", _AttributeType.dynamic_string),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
             ]
         )
         .set_propagate(),
@@ -338,10 +347,7 @@ class _Factory:
             ]
         ),
         "selector": lambda gui, control_type, attrs: _Builder(
-            gui=gui,
-            control_type=control_type,
-            element_name="Selector",
-            attributes=attrs,
+            gui=gui, control_type=control_type, element_name="Selector", attributes=attrs, default_value=None
         )
         .set_value_and_default(with_default=False, var_type=_AttributeType.lov_value)
         .get_adapter("lov")  # need to be called before set_lov
@@ -357,7 +363,8 @@ class _Factory:
                 ("value_by_id", _AttributeType.boolean),
                 ("multiple", _AttributeType.boolean),
                 ("width", _AttributeType.string_or_number),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
+                ("label",),
             ]
         )
         .set_refresh_on_update()
@@ -370,6 +377,7 @@ class _Factory:
             default_value=0,
         )
         .set_value_and_default(native_type=True, var_type=_AttributeType.number_or_lov_value)
+        .set_change_delay()
         .set_attributes(
             [
                 ("active", _AttributeType.dynamic_boolean, True),
@@ -381,7 +389,8 @@ class _Factory:
                 ("min", _AttributeType.number, 0),
                 ("orientation"),
                 ("width", _AttributeType.string, "300px"),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
+                ("continuous", _AttributeType.boolean, True),
             ]
         )
         .get_adapter("lov")  # need to be called before set_lov
@@ -464,7 +473,7 @@ class _Factory:
                 ("label",),
                 ("value_by_id", _AttributeType.boolean),
                 ("unselected_value", _AttributeType.string, ""),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
             ]
         )
         .set_kind()
@@ -490,7 +499,9 @@ class _Factory:
                 ("value_by_id", _AttributeType.boolean),
                 ("multiple", _AttributeType.boolean),
                 ("width", _AttributeType.string_or_number),
-                ("on_change", ),
+                ("on_change", _AttributeType.function),
+                ("select_leafs_only", _AttributeType.boolean),
+                ("row_height", _AttributeType.string),
             ]
         )
         .set_refresh_on_update()
